import org.junit.jupiter.api.*;
import org.junit.jupiter.api.io.TempDir;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

class MainTest {

	// File handling variables
	@TempDir
	private static Path tempDir;
	private static Path tempFile;
	
	// PrintStream (to capture the console output)
	private final ByteArrayOutputStream outContent = new ByteArrayOutputStream();
	private final ByteArrayOutputStream errContent = new ByteArrayOutputStream();
	
	// Save the old PrintStreams
	private final PrintStream oldOut = System.out;
	private final PrintStream oldErr = System.err;
	
	// Argument handling
	private String[] arguments;
	
	@BeforeAll
	static void start() throws IOException {
		tempFile = Files.createFile(tempDir.resolve("testing.txt"));
	}
	
	@BeforeEach
	void setUp() {
		// Create new PrintStream to capture console outputs
		System.setOut(new PrintStream(outContent));
		System.setErr(new PrintStream(errContent));
		
		// Reset arguments to new array with only 1 item
		arguments = new String[1];
	}
	
	@AfterEach
	void tearDown() {
		// Reset PrintStream to original
		System.setOut(oldOut);
		System.setErr(oldErr);
	}
	
	@Test
	@DisplayName("No arguments provided")
	void noArguments() {
		arguments = new String[0];
		
		Main.main(arguments);
		
		// Check that the program quit at the expected point, with expected output
		assertEquals("Invalid arguments, you need to define the text file to read from!\n", outContent.toString());
	}
	
	@Test
	@DisplayName("Too many arguments provided")
	void tooManyArguments() {
		arguments = new String[2];
		// Simulating a potential typo of the path with a space
		arguments[0] = "/path/to/";
		arguments[1] = "file.txt";
		
		Main.main(arguments);

		// Check that the program quit at the expected point, with expected output
		assertEquals("Invalid arguments, you need to define the text file to read from!\n", outContent.toString());
	}
	
	@Test
	@DisplayName("Invalid/Nonexistent file")
	void invalidFile() {
		arguments = new String[1];
		
		arguments[0] = "doesNotExist.txt";
		
		assertThrowsExactly(RuntimeException.class, () -> Main.main(arguments));
	}

	@Test
	@DisplayName("Normal test run")
	void normalRun() throws IOException {
		// List of emails to add to the file
		List<String> emails = new ArrayList<>() {{
			add("johndoe@gmail.com");
			add("janedoe@gmail.com");
			add("person@outlook.com");
			add("totallylegitpresident@gmail.com");
			add("google@yahoo.com");
			add("person1st@outlook.com");
			add("google+scam@gmail.com");
		}};
		
		Files.write(tempFile, emails);
		
		// The expected output - \n has been added on the end due to the program quitting at the end
		String expectedOutput = "gmail.com 4\noutlook.com 2\nyahoo.com 1\n";
		
		// Add argument for filepath
		arguments[0] = tempFile.toString();
		
		Main.main(arguments);
		
		assertEquals(expectedOutput, outContent.toString());
	}
	
	@Test
	@DisplayName("File with invalid emails")
	void brokenData() throws IOException {
		// List of emails, with some of the emails being invalid
		List<String> emails = new ArrayList<>() {{
			add("gmail.com");
			add("janedoe@gmail.com");
			add("person@outlook.com");
			add("totallylegit@president@gmail.com");
			add("google@yahoo");
			add("person1st@outlook,com");
			add("google+scam@gmail.com");
		}};

		Files.write(tempFile, emails);

		// Add argument for filepath
		arguments[0] = tempFile.toString();
		
		// Checks for the fail from the email validation
		assertThrowsExactly(IllegalArgumentException.class, () -> Main.main(arguments));
	}
}